<?php

namespace WcDPD;

defined('ABSPATH') || exit;

class DpdParcelShopShippingMethod extends \WC_Shipping_Method
{
    public const SETTINGS_ID_KEY = 'wc_dpd_parcelshop';
    public const SHIPPING_PRICE_TYPE_OPTION_KEY = 'wc_dpd_shipping_price_type';
    public const FREE_FIXED_SHIPPING_OPTION_KEY = 'wc_dpd_free_shipping_price';
    public const PRODUCTS_WEIGHT_SHIPPING_RATES_OPTION_KEY = 'wc_dpd_products_weight_shipping_rates';
    public const PARCELSHOP_ID_META_KEY = 'wc_dpd_parcelshop_id';
    public const PARCELSHOP_NAME_META_KEY = 'wc_dpd_parcelshop_name';
    public const PARCELSHOP_STREET_META_KEY = 'wc_dpd_parcelshop_street';
    public const PARCELSHOP_ZIP_META_KEY = 'wc_dpd_parcelshop_zip';
    public const PARCELSHOP_CITY_META_KEY = 'wc_dpd_parcelshop_city';
    public const PARCELSHOP_COUNTRY_CODE_META_KEY = 'wc_dpd_parcelshop_country_code';
    public const PARCELSHOP_COUNTRY_NAME_META_KEY = 'wc_dpd_parcelshop_country_name';

    /**
     * Constructor for the shipping class
     *
     * @param integer $instance_id
     */
    public function __construct(int $instance_id = 0)
    {
        parent::__construct();

        $this->id = self::SETTINGS_ID_KEY;
        $this->instance_id = absint($instance_id);
        $this->method_title = __('DPD parcelshop', 'wc-dpd');
        $this->method_description = __('Allow customers to deliver to the DPD parcelshops.', 'wc-dpd');
        $this->supports = [
            'shipping-zones',
            'instance-settings',
            'instance-settings-modal',
        ];

        $this->init();
    }

    /**
     * Init function
     *
     * @return void
     */
    public function init()
    {
        // Load the settings.
        $this->init_form_fields();
        $this->init_settings();

        // Define user set variables.
        $this->title = __('DPD Pickup/Pickup Station', 'wc-dpd');
        $this->tax_status = $this->get_option('tax_status');

        $fee = (float) wp_kses_post($this->get_option('fee'));
        $this->fee = $fee ? $fee : 0;

        $this->init_form_fields();
        $this->init_settings();

        add_action('admin_footer', [__CLASS__, 'addScripts'], 10, 1);
        add_filter('woocommerce_generate_repeater_html', [$this, 'addRepeaterFieldHtml'], 10, 4);

        add_filter('woocommerce_shipping_'.self::SETTINGS_ID_KEY.'_instance_settings_values', [$this, 'adjustPostData'], 0, 2);
        add_action('woocommerce_update_options_shipping_' . self::SETTINGS_ID_KEY, [$this, 'process_admin_options']);
    }

    /**
     * Initialize method options fields
     *
     * @return void
     */
    public function init_form_fields()
    {
        $weight_unit = (string) get_option('woocommerce_weight_unit');

        $this->instance_form_fields = [
            'tax_status' => array(
                'title'   => __('Tax status', 'wc-dpd'),
                'type'    => 'select',
                'class'   => 'wc-enhanced-select',
                'default' => 'none',
                'options' => array(
                    'none'    => _x('None', 'Tax status', 'wc-dpd'),
                    'taxable' => __('Taxable', 'wc-dpd'),
                ),
            ),
            self::SHIPPING_PRICE_TYPE_OPTION_KEY => [
                'title' => __('Shipping type', 'wc-dpd'),
                'type' => 'select',
                'options' => [
                    'fixed' => __('Fixed shipping price', 'wc-dpd'),
                    'products_weight_based' => __('Products weight based shipping price', 'wc-dpd'),
                ],
                'description' => __('Choose type of the shipping.', 'wc-dpd'),
                'desc_tip' => true,
                'class' => 'js-dpd-shipping-type-select'
            ],
            'fee' => [
                'title' => __('Delivery fee', 'wc-dpd'),
                'type' => 'price',
                'description' => __('What fee do you want to charge for shipping to the parcelshop.', 'wc-dpd'),
                'default' => '',
                'desc_tip' => true,
                'placeholder' => wc_format_localized_price(0),
                'class' => 'js-dpd-fixed-shipping-type'
            ],
            self::FREE_FIXED_SHIPPING_OPTION_KEY => [
                'title' => __('Free shipping from', 'wc-dpd'),
                'type' => 'price',
                'description' => __('If you exceed the price in the cart, shipping will be free.', 'wc-dpd'),
                'default' => '',
                'desc_tip' => true,
                'placeholder' => wc_format_localized_price(0),
                'class' => 'js-dpd-fixed-shipping-type'
            ],
            self::PRODUCTS_WEIGHT_SHIPPING_RATES_OPTION_KEY => [
                'title' => __('Products weight based shipping rates', 'wc-dpd'),
                'type' => 'repeater',
                'description' => __('Add shipping rates based on the weight of products in the cart.', 'wc-dpd'),
                'desc_tip' => true,
                'label_text' => __('Shipping rate', 'wc-dpd'),
                'min_weight_input_text' => sprintf(__('Min weight (%s)', 'wc-dpd'), $weight_unit),
                'max_weight_input_text' => sprintf(__('Max weight (%s)', 'wc-dpd'), $weight_unit),
                'price_input_text' => __('Price', 'wc-dpd') . ' ' . (wc_prices_include_tax() ? __('with', 'wc-dpd') : __('without', 'wc-dpd')). ' '.  __('tax', 'wc-dpd'),
                'min_weight_input_placeholder_text' => __('Min weight', 'wc-dpd'),
                'max_weight_input_placeholder_text' => __('Max weight', 'wc-dpd'),
                'price_input_placeholder_text' => __('Price', 'wc-dpd'),
                'add_btn_text' => __('Add a shipping rate', 'wc-dpd'),
                'class' => 'js-dpd-weight-based-shipping-type'
            ]
        ];
    }

    /**
     * Calculate shipping cost
     *
     * @param array $package
     *
     * @return void
     */
    public function calculate_shipping($package = [])
    {
        $free_shipping = isset($this->instance_settings[self::FREE_FIXED_SHIPPING_OPTION_KEY]) && (float) $this->instance_settings[self::FREE_FIXED_SHIPPING_OPTION_KEY] > 0 ? (float) $this->instance_settings[self::FREE_FIXED_SHIPPING_OPTION_KEY] : 0;
        $cart_total = WC()->cart->get_cart_contents_total();

        $rate = [
            'id' => $this->id,
            'label' => $this->title,
            'cost' => $free_shipping && $cart_total > $free_shipping ? 0 : (float) $this->fee,
            'calc_tax' => 'per_order'
        ];

        // Maybe calculate Products weight based shipping
        if(
            isset($this->instance_settings[self::SHIPPING_PRICE_TYPE_OPTION_KEY]) &&
            $this->instance_settings[self::SHIPPING_PRICE_TYPE_OPTION_KEY] == 'products_weight_based' &&
            !empty($this->instance_settings[self::PRODUCTS_WEIGHT_SHIPPING_RATES_OPTION_KEY])
        ) {
            $items_total_weight = self::getCartTotalWeight();
            $products_weight_based_rates = maybe_unserialize($this->instance_settings[self::PRODUCTS_WEIGHT_SHIPPING_RATES_OPTION_KEY]);

            if(!empty($products_weight_based_rates)) {
                foreach ($products_weight_based_rates as $products_weight_rate) {
                    $weight_rate_from = !empty($products_weight_rate['min']) ? (int) $products_weight_rate['min'] : 0;
                    $weight_rate_to = !empty($products_weight_rate['max']) ? (int) $products_weight_rate['max'] : 999;

                    if($items_total_weight >= $weight_rate_from) {
                        $rate['cost'] = !empty($products_weight_rate['price']) ? number_format((float) $products_weight_rate['price'], wc_get_price_decimals(), '.', '') : 0;
                    }

                    if ($items_total_weight >= $weight_rate_from && $items_total_weight <= $weight_rate_to) {
                        $rate['cost'] = !empty($products_weight_rate['price']) ? number_format((float) $products_weight_rate['price'], wc_get_price_decimals(), '.', '') : 0;
                    }
                }
            }
        }

        $this->add_rate($rate);
    }

    /**
     * Get total weight of products in cart
     *
     * @return float|int
     */
    public static function getCartTotalWeight()
    {
        $cart = WC()->cart;

        if(!$cart) {
            return 0;
        }

        // Get an array of cart item objects
        $cart_items = $cart->get_cart();

        if(!$cart_items) {
            return 0;
        }

        // Initialize the total weight variable
        $total_weight = 0;

        // Loop through each cart item and add its weight to the total weight
        foreach ($cart_items as $cart_item_key => $cart_item) {
            $product = !empty($cart_item['data']) ? $cart_item['data'] : null;

            if(!$product instanceof \WC_Product) {
                continue;
            }

            $product_weight = (float) $product->get_weight();

            if(!$product_weight) {
                continue;
            }

            $quantity = !empty($cart_item['quantity']) ? (int) $cart_item['quantity'] : 1;
            $total_weight += ($product_weight * $quantity);
        }

        return (float) $total_weight;
    }

    /**
     * Adjust repeater field values on save
     */
    public function adjustPostData($settings, $instance)
    {
        if (empty($_POST['data'])) {
            return;
        }

        $repeater_values = [];

        $post_data = $_POST['data'];

        if (!empty($post_data['woocommerce_'.self::SETTINGS_ID_KEY.'_'.self::PRODUCTS_WEIGHT_SHIPPING_RATES_OPTION_KEY.'_min'])) {
            foreach ($post_data['woocommerce_'.self::SETTINGS_ID_KEY.'_'.self::PRODUCTS_WEIGHT_SHIPPING_RATES_OPTION_KEY.'_min'] as $key => $value) {
                $repeater_values[$key]['min'] = (int) sanitize_text_field($value);
            }

            unset($post_data['woocommerce_'.self::SETTINGS_ID_KEY.'_'.self::PRODUCTS_WEIGHT_SHIPPING_RATES_OPTION_KEY.'_min']);
        }

        if (!empty($post_data['woocommerce_'.self::SETTINGS_ID_KEY.'_'.self::PRODUCTS_WEIGHT_SHIPPING_RATES_OPTION_KEY.'_max'])) {
            foreach ($post_data['woocommerce_'.self::SETTINGS_ID_KEY.'_'.self::PRODUCTS_WEIGHT_SHIPPING_RATES_OPTION_KEY.'_max'] as $key => $value) {
                $repeater_values[$key]['max'] = (int) sanitize_text_field($value);
            }

            unset($post_data['woocommerce_'.self::SETTINGS_ID_KEY.'_'.self::PRODUCTS_WEIGHT_SHIPPING_RATES_OPTION_KEY.'_max']);
        }

        if (!empty($post_data['woocommerce_'.self::SETTINGS_ID_KEY.'_'.self::PRODUCTS_WEIGHT_SHIPPING_RATES_OPTION_KEY.'_price'])) {
            foreach ($post_data['woocommerce_'.self::SETTINGS_ID_KEY.'_'.self::PRODUCTS_WEIGHT_SHIPPING_RATES_OPTION_KEY.'_price'] as $key => $value) {
                $repeater_values[$key]['price'] = number_format((float) $value, wc_get_price_decimals(), '.', '');
            }

            unset($post_data['woocommerce_'.self::SETTINGS_ID_KEY.'_'.self::PRODUCTS_WEIGHT_SHIPPING_RATES_OPTION_KEY.'_price']);
        }

        $settings[self::PRODUCTS_WEIGHT_SHIPPING_RATES_OPTION_KEY] = serialize($repeater_values);

        return $settings;
    }

     /**
     * Add repeater field html
     *
     * @param string $html
     * @param string $key
     * @param array $data
     * @param object $wc_settings
     *
     * @return string
     */
    public function addRepeaterFieldHtml($html = '', $key = '', $data = [], $wc_settings = null)
    {
        if($key != self::PRODUCTS_WEIGHT_SHIPPING_RATES_OPTION_KEY) {
            return $html;
        }

        $field_key = $this->get_field_key($key);

        $defaults  = array(
            'title' => '',
            'disabled' => false,
            'class' => '',
            'label_text' => '',
            'desc_tip' => false,
            'type' => 'repeater',
            'add_btn_text' => '',
            'description' => '',
            'repeater_description' => '',
            'min_weight_input_text' => '',
            'max_weight_input_text' => '',
            'price_input_text' => '',
            'min_weight_input_placeholder_text' => '',
            'max_weight_input_placeholder_text' => '',
            'price_input_placeholder_text' => '',
        );

        $data = wp_parse_args($data, $defaults);

        $values = self::getRepeaterOptions($key, $wc_settings);
        $values = htmlspecialchars(json_encode($values), ENT_QUOTES, 'UTF-8');

        $props = [
            'minWeightInputText' => $data['min_weight_input_text'],
            'maxWeightInputText' => $data['max_weight_input_text'],
            'priceInputText' => $data['price_input_text'],
            'minWeightInputPlaceholderText' => $data['min_weight_input_placeholder_text'],
            'maxWeightInputPlaceholderText' => $data['max_weight_input_placeholder_text'],
            'priceInputPlaceholderText' => $data['price_input_placeholder_text'],
            'inputName' => $field_key,
            'labelText' => $data['label_text'],
            'removeLabel' => __('Remove', 'wc-dpd'),
            'title' => __('Title', 'wc-dpd'),
        ];
        $props = htmlspecialchars(json_encode($props), ENT_QUOTES, 'UTF-8');

        ob_start();
        ?>
			<tr valign="top">
				<th scope="row" class="titledesc">
					<label for="<?php echo esc_attr($field_key); ?>"><?php echo wp_kses_post($data['title']); ?> <?php echo $this->get_tooltip_html($data); // WPCS: XSS ok.?></label>
				</th>

				<td class="forminp">
					<fieldset class="repeatable-field repeatable-field--<?php echo $key; ?> <?php echo esc_attr($data['class']); ?>" data-component="field-repeater" data-props="<?php echo $props; ?>" data-inputs-data="<?php echo $values; ?>" tabindex="0">
						<legend class="screen-reader-text"><span><?php echo wp_kses_post($data['title']); ?></span></legend>

						<?php if (!empty($data['repeater_description'])) : ?>
							<p><small><?php echo wp_kses_post($data['repeater_description']); ?></small></p>
						<?php endif; ?>

						<ol class="repeatable-field__rows" data-ref="rowList"></ol>

						<div class="repeatable-field__bottom">
							<button class="repeatable-field__add-button button" data-ref="addButton" type="button">+ <?php echo esc_attr($data['add_btn_text']); ?></button>
						</div>

						<?php echo $this->get_description_html($data); // WPCS: XSS ok.?>
					</fieldset>
				</td>
			</tr>
		<?php

        return ob_get_clean();
    }

    /**
     * Get repeater option list of options
     *
     * @param string $option_key
     * @param object $wc_settings
     *
     * @return array
     */
    public static function getRepeaterOptions($option_key = '', $wc_settings = null)
    {
        return maybe_unserialize($wc_settings->get_option($option_key));
    }

    /**
     * Add admin scripts
     *
     * @return void
     */
    public static function addScripts()
    {
        global $pagenow;

        if ($pagenow !== 'admin.php') {
            return;
        }

        $page = isset($_GET['page']) ? sanitize_text_field($_GET['page']) : '';
        $tab  = isset($_GET['tab']) ? sanitize_text_field($_GET['tab']) : '';

        if ($page !== 'wc-settings' || $tab !== 'shipping') {
            return;
        }

        // Repeater field assets
        wp_enqueue_script(self::SETTINGS_ID_KEY . '_repeater_field', WCDPD_PLUGIN_ASSETS_URL . 'scripts/dpd-parcelshop-shipping-method-weight-by-package-repeater.js', [], wc_dpd_get_plugin_version(), true);
        wp_enqueue_style(self::SETTINGS_ID_KEY . '_repeater_field', WCDPD_PLUGIN_ASSETS_URL . 'styles/dpd-export-repeater-settings-field.css', [], wc_dpd_get_plugin_version(), 'all');
    }
}
