<?php

namespace WcDPD;

defined('ABSPATH') || exit;

/**
 * Order class
 */
class Order
{
    public const BANK_ID_META_KEY = 'dpd_bank_id';
    public const ADDRESS_ID_META_KEY = 'dpd_address_id';
    public const SHIPPING_META_KEY = 'dpd_shipping';
    public const NOTIFICATION_META_KEY = 'dpd_notification';
    public const REFERENCE_1_META_KEY = 'dpd_refrence_1';
    public const REFERENCE_2_META_KEY = 'dpd_refrence_2';
    public const EXPORT_STATUS_META_KEY = 'dpd_export_status';
    public const EXPORT_SUCCESS_STATUS = 'success';
    public const EXPORT_FAILED_STATUS = 'failed';
    public const EXPORT_LABEL_URL_META_KEY = 'dpd_export_label_url';
    public const EXPORT_PACKAGE_NUMBER_META_KEY = 'dpd_export_package_number';
    public const EXPORT_MPSID_META_KEY = 'dpd_export_mpsid';

    public static function init()
    {
        add_action('woocommerce_checkout_update_order_meta', [__CLASS__, 'saveParcelShopShippingMethodFieldsToOrder'], 10, 2);
        add_action('woocommerce_order_details_after_order_table', [__CLASS__, 'displayParcelShopShippingOrderTableInfo'], 10, 1);
        add_action('woocommerce_admin_order_data_after_billing_address', [__CLASS__, 'displayParcelShopShippingAdminOrderInfo'], 10, 1);
    }

    /**
     * Export order to DPD
     *
     * @param integer $order_id
     *
     * @return bool
     */
    public static function export($order_id = 0)
    {
        if (!$order_id) {
            return false;
        }

        if (!self::canExportOrder($order_id)) {
            Notice::error(sprintf(__('Order %d is already exported to DPD.', 'wc-dpd'), $order_id));

            return false;
        }

        $order = wc_get_order($order_id);

        if (!$order instanceof \WC_Order) {
            return false;
        }

        $data = self::getOrderExportData($order);

        try {
            $response = DpdExport::doExport($data);

            update_post_meta($order_id, Order::EXPORT_LABEL_URL_META_KEY, $response[Order::EXPORT_LABEL_URL_META_KEY]);
            update_post_meta($order_id, Order::EXPORT_PACKAGE_NUMBER_META_KEY, $response[Order::EXPORT_PACKAGE_NUMBER_META_KEY]);
            update_post_meta($order_id, Order::EXPORT_MPSID_META_KEY, $response[Order::EXPORT_MPSID_META_KEY]);
            update_post_meta($order_id, Order::EXPORT_STATUS_META_KEY, $response[Order::EXPORT_STATUS_META_KEY]);

            $message = sprintf(__('Order %d was successfully exported', 'wc-dpd'), $order_id);

            Notice::success($message);
            $order->add_order_note(Notice::PREFIX . $message);

            return true;
        } catch (\Exception $e) {
            $message = esc_html($e->getMessage());

            Notice::error($message);
            $order->add_order_note(Notice::PREFIX . $message);

            return false;
        }
    }

    /**
     * Get order data for export
     *
     * @param \WC_Order $order
     *
     * @return array
     */
    public static function getOrderExportData(\WC_Order $order)
    {
        $order_id = $order->get_id();
		$order_number = $order->get_order_number();
        $shipment_type = 'b2c';

        $billing_full_name = $order->get_formatted_billing_full_name();
        $shipping_full_name = $order->get_formatted_shipping_full_name();
        $full_name = trim($shipping_full_name) ? $shipping_full_name : $billing_full_name;

        $billing_company = $order->get_billing_company();
        $shipping_company = $order->get_shipping_company();
        $company = trim($shipping_company) ? $shipping_company : $billing_company;

        if ($company) {
            $shipment_type = 'b2b';
        }

        $billing_address_1 = $order->get_billing_address_1();
        $billing_address_2 = $order->get_billing_address_2();
        $shipping_address_1 = $order->get_shipping_address_1();
        $shipping_address_2 = $order->get_shipping_address_2();

        $street = $shipping_address_1 ? $shipping_address_1 : $billing_address_1;
        $street_2 = $shipping_address_2 ? $shipping_address_2 : $billing_address_2;

        $house_number = '';
        if ($street_2) {
            $house_number = $street_2;
        } elseif (preg_match('/^([^\d]*[^\d\s]) *(\d.*)$/', $street, $parsed_address_1)) {
            $street = !empty($parsed_address_1[1]) ? $parsed_address_1[1] : '';
            $house_number = !empty($parsed_address_1[2]) ? $parsed_address_1[2] : '';
        }

        $billing_postcode = $order->get_billing_postcode();
        $shipping_postcode = $order->get_shipping_postcode();
        $postcode = $shipping_postcode ? $shipping_postcode : $billing_postcode;

        $billing_city = $order->get_billing_city();
        $shipping_city = $order->get_shipping_city();
        $city = $shipping_city ? $shipping_city : $billing_city;

        $billing_country_code = $order->get_billing_country();
        $shipping_country_code = $order->get_shipping_country();
        $country_code = $shipping_country_code ? $shipping_country_code : $billing_country_code;

        $phone = $order->get_billing_phone();
        $email = $order->get_billing_email();
        $customer_note = $order->get_customer_note();
        $order_price = $order->get_total();
        $order_currency = $order->get_currency();
        $shipping_method = $order->get_shipping_method();
        $payment_method = $order->get_payment_method();
        $date_now = wp_date('Ymd');
        $shipping = get_post_meta($order_id, self::SHIPPING_META_KEY, true);
        $parcelshop_id = 0;

        // If order has parcel shipping selected, change settings
        $has_parcelshop_shipping = Order::hasParcelShpping($order);
        if ($has_parcelshop_shipping) {
            $shipping = '17';
            $shipment_type = 'psd';
            $parcelshop_id = get_post_meta($order_id, DpdParcelShopShippingMethod::PARCELSHOP_ID_META_KEY, true);
        }

        $bank_id = sanitize_text_field(get_post_meta($order_id, self::BANK_ID_META_KEY, true));
        $address_id = sanitize_text_field(get_post_meta($order_id, self::ADDRESS_ID_META_KEY, true));
        $reference_1 = sanitize_text_field(get_post_meta($order_id, self::REFERENCE_1_META_KEY, true));
        $reference_2 = sanitize_text_field(get_post_meta($order_id, self::REFERENCE_2_META_KEY, true));

        $notification = get_post_meta($order_id, self::NOTIFICATION_META_KEY, true);
        $notification = $notification == 'yes' ? 'yes' : 'no';


        return [
            DpdExport::SHIPMENT_TYPE_KEY => $shipment_type,
            DpdExport::CUSTOMER_FULL_NAME_KEY => $full_name,
            DpdExport::CUSTOMER_COMPANY_KEY => $company,
            DpdExport::CUSTOMER_STREET_KEY => $street,
            DpdExport::CUSTOMER_HOUSE_NUMBER_KEY => $house_number,
            DpdExport::CUSTOMER_ZIP_KEY  => $postcode,
            DpdExport::CUSTOMER_CITY_KEY  => $city,
            DpdExport::CUSTOMER_COUNTRY_KEY => $country_code,
            DpdExport::CUSTOMER_PHONE_KEY => $phone,
            DpdExport::CUSTOMER_EMAIL_KEY => $email,
            DpdExport::ORDER_NOTE_KEY => $order_number,
            DpdExport::ORDER_REFERENCE_1_KEY => $reference_1,
            DpdExport::ORDER_REFERENCE_2_KEY => $reference_2,
            DpdExport::ORDER_ID_KEY => $order_id,
            DpdExport::ORDER_PRICE_KEY => $order_price,
            DpdExport::ORDER_CURRENCY_KEY => $order_currency,
            DpdExport::ORDER_PAYMENT_METHOD_KEY => $payment_method,
            DpdExport::ORDER_HAS_PARCELSHOP_SHIPPING_KEY => $has_parcelshop_shipping,
            DpdExport::ORDER_SHIPPING_METHOD_KEY => $shipping_method,
            DpdExport::ORDER_PARCELSHOP_ID => $parcelshop_id,
            DpdExport::DATE_NOW_KEY => $date_now,
            DpdExportSettings::SHIPPING_OPTION_KEY => $shipping,
            DpdExportSettings::ADDRESS_ID_OPTION_KEY => $address_id,
            DpdExportSettings::BANK_ID_OPTION_KEY => $bank_id,
            DpdExportSettings::NOTIFICATION_OPTION_KEY => $notification,
        ];
    }

    /**
     * Check if order can be exported to DPD
     *
     * @param int $order_id
     *
     * @return bool
     */
    public static function canExportOrder($order_id)
    {
        $export_status = get_post_meta($order_id, self::EXPORT_STATUS_META_KEY, true);

        if ($export_status == self::EXPORT_SUCCESS_STATUS) {
            return false;
        }

        return true;
    }

    /**
     * Save parcelshop fields to the order
     *
     * @param int $order_id
     *
     * @return void
     */
    public static function saveParcelShopShippingMethodFieldsToOrder($order_id)
    {
        if (is_admin()) {
            return;
        }

        if (empty($_POST['shipping_method'][0])) {
            return;
        }

        if ($_POST['shipping_method'][0] != DpdParcelShopShippingMethod::SETTINGS_ID_KEY) {
            return;
        }

        $parcelshop_id = !empty($_POST[DpdParcelShopShippingMethod::PARCELSHOP_ID_META_KEY]) ? (int) wp_kses_post(trim($_POST[DpdParcelShopShippingMethod::PARCELSHOP_ID_META_KEY])) : '';
        $parcelshop_name = !empty($_POST[DpdParcelShopShippingMethod::PARCELSHOP_NAME_META_KEY]) ? (string) wp_kses_post(trim($_POST[DpdParcelShopShippingMethod::PARCELSHOP_NAME_META_KEY])) : '';
        $parcelshop_street = !empty($_POST[DpdParcelShopShippingMethod::PARCELSHOP_STREET_META_KEY]) ? (string) wp_kses_post(trim($_POST[DpdParcelShopShippingMethod::PARCELSHOP_STREET_META_KEY])) : '';
        $parcelshop_zip = !empty($_POST[DpdParcelShopShippingMethod::PARCELSHOP_ZIP_META_KEY]) ? (string) wp_kses_post(trim($_POST[DpdParcelShopShippingMethod::PARCELSHOP_ZIP_META_KEY])) : '';
        $parcelshop_city = !empty($_POST[DpdParcelShopShippingMethod::PARCELSHOP_CITY_META_KEY]) ? (string) wp_kses_post(trim($_POST[DpdParcelShopShippingMethod::PARCELSHOP_CITY_META_KEY])) : '';
        $parcelshop_country_code = !empty($_POST[DpdParcelShopShippingMethod::PARCELSHOP_COUNTRY_CODE_META_KEY]) ? (string) wp_kses_post(trim($_POST[DpdParcelShopShippingMethod::PARCELSHOP_COUNTRY_CODE_META_KEY])) : '';

        update_post_meta($order_id, DpdParcelShopShippingMethod::PARCELSHOP_ID_META_KEY, $parcelshop_id);
        update_post_meta($order_id, DpdParcelShopShippingMethod::PARCELSHOP_NAME_META_KEY, $parcelshop_name);
        update_post_meta($order_id, DpdParcelShopShippingMethod::PARCELSHOP_STREET_META_KEY, $parcelshop_street);
        update_post_meta($order_id, DpdParcelShopShippingMethod::PARCELSHOP_ZIP_META_KEY, $parcelshop_zip);
        update_post_meta($order_id, DpdParcelShopShippingMethod::PARCELSHOP_CITY_META_KEY, $parcelshop_city);
        update_post_meta($order_id, DpdParcelShopShippingMethod::PARCELSHOP_COUNTRY_CODE_META_KEY, $parcelshop_country_code);

        // Clear chosen parcelshop session data
        WC()->session->set(Shipping::SESSION_CHOSEN_PARCELSHOP_KEY, []);
    }

    /**
     * Display parcelshop shipping info after order detail
     *
     * @param object $order
     *
     * @return void
     */
    public static function displayParcelShopShippingOrderTableInfo(object $order)
    {
        echo self::getParcelShopOrderHtmlDetails($order);
    }

    /**
     * Display parcelshop shipping info in the admin order detail
     *
     * @param object $order
     *
     * @return void
     */
    public static function displayParcelShopShippingAdminOrderInfo(object $order)
    {
        echo self::getParcelShopOrderHtmlDetails($order, 'admin');
    }

    /**
     * Get parcelshop order html details
     *
     * @param \WC_Order $order
     * @param string $type
     *
     * @return string
     */
    public static function getParcelShopOrderHtmlDetails($order, $type = '')
    {
        if (!$order instanceof \WC_Order) {
            return;
        }


        $order_id = (int) $order->get_ID();
        $has_parcelshop_shipping_method = self::hasParcelShpping($order);

        if (!$has_parcelshop_shipping_method) {
            return;
        }

        $parcelshop_id = (int) get_post_meta($order_id, DpdParcelShopShippingMethod::PARCELSHOP_ID_META_KEY, true);
        $parcelshop_name = (string) get_post_meta($order_id, DpdParcelShopShippingMethod::PARCELSHOP_NAME_META_KEY, true);
        $parcelshop_street = (string) get_post_meta($order_id, DpdParcelShopShippingMethod::PARCELSHOP_STREET_META_KEY, true);
        $parcelshop_zip = (string) get_post_meta($order_id, DpdParcelShopShippingMethod::PARCELSHOP_ZIP_META_KEY, true);
        $parcelshop_city = (string) get_post_meta($order_id, DpdParcelShopShippingMethod::PARCELSHOP_CITY_META_KEY, true);
        $parcelshop_country_code = (string) get_post_meta($order_id, DpdParcelShopShippingMethod::PARCELSHOP_COUNTRY_CODE_META_KEY, true);

        $countries = (array) WC()->countries->get_allowed_countries();
        $parcelshop_country_name = isset($countries[strtoupper($parcelshop_country_code)]) ? (string) $countries[strtoupper($parcelshop_country_code)] : '';

        return include_template('chosen-parcelshop-order-data.php', [
            'type' => $type,
            'parcelshop_id' => $parcelshop_id,
            'parcelshop_name' => $parcelshop_name,
            'parcelshop_street' => $parcelshop_street,
            'parcelshop_zip' => $parcelshop_zip,
            'parcelshop_city' => $parcelshop_city,
            'parcelshop_country_name' => $parcelshop_country_name,
            'parcelshop_country_code' => $parcelshop_country_code,
        ]);
    }

    /**
     * Check if order has parcel shipping method selected
     *
     * @param \WC_Order $order
     *
     * @return boolean
     */
    public static function hasParcelShpping(\WC_Order $order)
    {
        $order_shipping_methods = (array) $order->get_shipping_methods();

        foreach ($order_shipping_methods as $_key => $shipping_method) {
            if ($shipping_method->get_method_id() !== DpdParcelShopShippingMethod::SETTINGS_ID_KEY) {
                continue;
            }

            return true;
        }

        return false;
    }

    /**
     * Bulk download labels
     *
     * @param array $order_ids
     *
     * @return bool
     */
    public static function bulkDownloadLabels($order_ids = [])
    {
        if (empty($order_ids)) {
            Notice::error('You have to select at least one order.');

            return false;
        }

        $package_numbers = [];
        $processing_order_ids = [];
        foreach ($order_ids as $order_id) {
            $package_number = wp_kses_post(get_post_meta((int) $order_id, self::EXPORT_PACKAGE_NUMBER_META_KEY, true));

            if (!$package_number) {
                Notice::error(sprintf(__('Order %d does not have the package number and its label couldn\'t be printed.', 'wc-dpd'), $order_id));

                continue;
            }

            $package_numbers[] = $package_number;
            $processing_order_ids[] = $order_id;
        }

        if (empty($package_numbers)) {
            Notice::error(sprintf(__('None of your selected orders have a package number.', 'wc-dpd'), $order_id));

            return false;
        }

        $client = new Client();
        $pdf_content = $client->bulkDownloadLabels($package_numbers);

        if (!$pdf_content) {
            Notice::error(sprintf(__('Something went wrong and the pdf content is not valid. Please check orders %s if the package numbers are correct.', 'wc-dpd'), implode(', ', $processing_order_ids)));

            return false;
        }

        // Generate pdf
        header('Content-type: application/pdf');
        header('Content-Disposition: attachment; filename="labels.pdf"');
        echo $pdf_content;

        exit;
    }
}
