<?php

namespace WcDPD;

defined('ABSPATH') || exit;

/**
 * Log to the file
 *
 * @param string $message
 * @param mixed $variable
 *
 * @return void
 */
function wc_dpd_log($message, $variable = null)
{
    $message = trim($message);
    $message = "DEBUG: $message";

    $file = wc_get_log_file_path('wc_dpd');
    $date = get_date_from_gmt(gmdate('Y-m-d H:i:s'), 'Y-m-d H:i:s O');

    if (null !== $variable) {
        $message .= ': ' . wc_dpd_dump($variable);
    }

    // implementation note:
    // we do not use WooCommerce log handler intentionally, because it does not
    // use file locking, thus there is possibility that log gets damaged
    file_put_contents($file, "[$date] $message\n", LOCK_EX | FILE_APPEND);
}

/**
 * Dump variable
 *
 * @param mixed $variable
 *
 * @return mixed
 */
function wc_dpd_dump($variable)
{
    $type = gettype($variable);
    switch ($type) {
        case 'boolean':
            return $variable ? 'true' : 'false';
        case 'integer':
            return strval($variable);
        case 'double':
            return sprintf('%.1f', $variable);
        case 'string':
            $variable = str_replace([ '\\', "'" ], [ '\\\\', "\\'" ], $variable);
            return "'" . preg_replace_callback('/[\x00-\x1f\x7f-\xff]/', function ($m) {
                return "\\x" . bin2hex($m[0]);
            }, $variable) . "'";
        case 'array':
            $items = [];
            foreach ($variable as $key => $value) {
                $items[] = wc_dpd_dump($key) . ': ' . wc_dpd_dump($value);
            }
            return $items ? '[ ' . implode(', ', $items) . ' ]' : '[]';
        case 'object':
            $items = [];
            foreach (get_object_vars($variable) as $key => $value) {
                $items[] = wc_dpd_dump($key) . ': ' . wc_dpd_dump($value);
            }
            return get_class($variable) . ' ' . ($items ? '{ ' . implode(', ', $items) . ' }' : '{}');
        case 'resource':
        case 'resource (closed)':
            return $type;
        case 'NULL':
            return 'null';
    }
    return "??? $type ???";
}

/**
 * Get current plugin version
 *
 * @return string
 */
function wc_dpd_get_plugin_version()
{
    $plugin_data = get_plugin_data(WCDPD_PLUGIN_INDEX);
    $plugin_version = !empty($plugin_data['Version']) ? (string) $plugin_data['Version'] : '1.0.0';

    return $plugin_version;
}

/**
 * Include php template and pass variables there
 *
 * @param string $file_path
 * @param array $variables
 * @param boolean $print
 *
 * @return string
 */
function include_template($file_path, $variables = [])
{
    $templates_root = WCDPD_PLUGIN_TEMPLATES_PATH . DIRECTORY_SEPARATOR;
    $output = null;

    if (file_exists($templates_root . $file_path)) {
        // Extract the variables to a local namespace
        extract($variables);

        // Start output buffering
        ob_start();

        // Include the template file
        include $templates_root . $file_path;

        // End buffering and return its contents
        $output = ob_get_clean();
    }

    return $output;
}
